@@ -160,6 +160,12 @@ void LLVMToSPIRVDbgTran::transLocationInfo() {
       unsigned LineNo = 0;
       unsigned Col = 0;
       for (const Instruction &I : BB) {
+        if (auto *II = dyn_cast<IntrinsicInst>(&I)) {
+          if (II->getIntrinsicID() == Intrinsic::dbg_label) {
+            // SPIR-V doesn't support llvm.dbg.label intrinsic translation
+            continue;
+          }
+        }
         const DebugLoc &DL = I.getDebugLoc();
         if (!DL.get()) {
           if (DbgScope || InlinedAt) { // Emit DebugNoScope
@@ -208,15 +214,6 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgEntry(const MDNode *DIEntry) {
   return Res;
 }
 
-template <typename T>
-SPIRVEntry *LLVMToSPIRVDbgTran::transDbgEntryRef(const TypedDINodeRef<T> &Ref,
-                                                 SPIRVEntry *Alternate) {
-  T *Resolved = Ref.resolve();
-  if (!Resolved && Alternate)
-    return Alternate;
-  return transDbgEntry(Resolved);
-}
-
 // Dispatcher implementation
 
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgEntryImpl(const MDNode *MDN) {
@@ -332,15 +329,6 @@ SPIRVType *LLVMToSPIRVDbgTran::getVoidTy() {
   return VoidT;
 }
 
-SPIRVEntry *LLVMToSPIRVDbgTran::getScope(DIScopeRef SR) {
-  if (DIScope *S = SR.resolve())
-    return transDbgEntry(S);
-  else {
-    assert(SPIRVCU && "Compilation unit must already be translated!");
-    return SPIRVCU;
-  }
-}
-
 SPIRVEntry *LLVMToSPIRVDbgTran::getScope(DIScope *S) {
   if (S)
     return transDbgEntry(S);
@@ -388,6 +376,10 @@ SPIRVWord mapDebugFlags(DINode::DIFlags DFlags) {
     Flags |= SPIRVDebug::FlagIsLValueReference;
   if (DFlags & DINode::FlagRValueReference)
     Flags |= SPIRVDebug::FlagIsRValueReference;
+  if (DFlags & DINode::FlagTypePassByValue)
+    Flags |= SPIRVDebug::FlagTypePassByValue;
+  if (DFlags & DINode::FlagTypePassByReference)
+    Flags |= SPIRVDebug::FlagTypePassByReference;
   return Flags;
 }
 
@@ -470,7 +462,7 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgBaseType(const DIBasicType *BT) {
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgPointerType(const DIDerivedType *PT) {
   using namespace SPIRVDebug::Operand::TypePointer;
   SPIRVWordVec Ops(OperandCount);
-  SPIRVEntry *Base = transDbgEntryRef(PT->getBaseType(), getVoidTy());
+  SPIRVEntry *Base = transDbgEntry(PT->getBaseType());
   Ops[BaseTypeIdx] = Base->getId();
   Ops[StorageClassIdx] = ~0U; // all ones denote no address space
   Optional<unsigned> AS = PT->getDWARFAddressSpace();
@@ -486,7 +478,7 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgPointerType(const DIDerivedType *PT) {
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgQualifiedType(const DIDerivedType *QT) {
   using namespace SPIRVDebug::Operand::TypeQualifier;
   SPIRVWordVec Ops(OperandCount);
-  SPIRVEntry *Base = transDbgEntryRef(QT->getBaseType());
+  SPIRVEntry *Base = transDbgEntry(QT->getBaseType());
   Ops[BaseTypeIdx] = Base->getId();
   Ops[QualifierIdx] = SPIRV::DbgTypeQulifierMap::map(
       static_cast<llvm::dwarf::Tag>(QT->getTag()));
@@ -496,7 +488,7 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgQualifiedType(const DIDerivedType *QT) {
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgArrayType(const DICompositeType *AT) {
   using namespace SPIRVDebug::Operand::TypeArray;
   SPIRVWordVec Ops(MinOperandCount);
-  SPIRVEntry *Base = transDbgEntryRef(AT->getBaseType());
+  SPIRVEntry *Base = transDbgEntry(AT->getBaseType());
   Ops[BaseTypeIdx] = Base->getId();
 
   DINodeArray AR(AT->getElements());
@@ -521,7 +513,7 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgTypeDef(const DIDerivedType *DT) {
   using namespace SPIRVDebug::Operand::Typedef;
   SPIRVWordVec Ops(OperandCount);
   Ops[NameIdx] = BM->getString(DT->getName())->getId();
-  SPIRVEntry *BaseTy = transDbgEntryRef(DT->getBaseType());
+  SPIRVEntry *BaseTy = transDbgEntry(DT->getBaseType());
   assert(BaseTy && "Couldn't translate base type!");
   Ops[BaseTypeIdx] = BaseTy->getId();
   Ops[SourceIdx] = getSource(DT)->getId();
@@ -547,7 +539,7 @@ LLVMToSPIRVDbgTran::transDbgSubroutineType(const DISubroutineType *FT) {
     // followed by types of the function arguments' types.
     // The same order is preserved in SPIRV.
     for (unsigned I = 0; I < NumElements; ++I)
-      Ops[ReturnTypeIdx + I] = transDbgEntryRef(Types[I], getVoidTy())->getId();
+      Ops[ReturnTypeIdx + I] = transDbgEntry(Types[I])->getId();
   } else { // void foo();
     Ops[ReturnTypeIdx] = getVoidTy()->getId();
   }
@@ -560,8 +552,8 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgEnumType(const DICompositeType *ET) {
   SPIRVWordVec Ops(MinOperandCount);
 
   SPIRVEntry *UnderlyingType = getVoidTy();
-  if (DITypeRef DerivedFrom = ET->getBaseType())
-    UnderlyingType = transDbgEntryRef(DerivedFrom);
+  if (DIType *DerivedFrom = ET->getBaseType())
+    UnderlyingType = transDbgEntry(DerivedFrom);
   ConstantInt *Size = getUInt(M, ET->getSizeInBits());
 
   Ops[NameIdx] = BM->getString(ET->getName())->getId();
@@ -640,11 +632,11 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgMemberType(const DIDerivedType *MT) {
   SPIRVWordVec Ops(MinOperandCount);
 
   Ops[NameIdx] = BM->getString(MT->getName())->getId();
-  Ops[TypeIdx] = transDbgEntryRef(MT->getBaseType())->getId();
+  Ops[TypeIdx] = transDbgEntry(MT->getBaseType())->getId();
   Ops[SourceIdx] = getSource(MT)->getId();
   Ops[LineIdx] = MT->getLine();
   Ops[ColumnIdx] = 0; // This version of DIDerivedType has no column number
-  Ops[ParentIdx] = transDbgEntryRef(MT->getScope())->getId();
+  Ops[ParentIdx] = transDbgEntry(MT->getScope())->getId();
   ConstantInt *Offset = getUInt(M, MT->getOffsetInBits());
   Ops[OffsetIdx] = SPIRVWriter->transValue(Offset, nullptr)->getId();
   ConstantInt *Size = getUInt(M, MT->getSizeInBits());
@@ -664,8 +656,8 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgMemberType(const DIDerivedType *MT) {
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgInheritance(const DIDerivedType *DT) {
   using namespace SPIRVDebug::Operand::TypeInheritance;
   SPIRVWordVec Ops(OperandCount);
-  Ops[ChildIdx] = transDbgEntryRef(DT->getScope())->getId();
-  Ops[ParentIdx] = transDbgEntryRef(DT->getBaseType())->getId();
+  Ops[ChildIdx] = transDbgEntry(DT->getScope())->getId();
+  Ops[ParentIdx] = transDbgEntry(DT->getBaseType())->getId();
   ConstantInt *Offset = getUInt(M, DT->getOffsetInBits());
   Ops[OffsetIdx] = SPIRVWriter->transValue(Offset, nullptr)->getId();
   ConstantInt *Size = getUInt(M, DT->getSizeInBits());
@@ -677,8 +669,8 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgInheritance(const DIDerivedType *DT) {
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgPtrToMember(const DIDerivedType *DT) {
   using namespace SPIRVDebug::Operand::PtrToMember;
   SPIRVWordVec Ops(OperandCount);
-  Ops[MemberTypeIdx] = transDbgEntryRef(DT->getBaseType())->getId();
-  Ops[ParentIdx] = transDbgEntryRef(DT->getClassType())->getId();
+  Ops[MemberTypeIdx] = transDbgEntry(DT->getBaseType())->getId();
+  Ops[ParentIdx] = transDbgEntry(DT->getClassType())->getId();
   return BM->addDebugInfo(SPIRVDebug::TypePtrToMember, getVoidTy(), Ops);
 }
 
@@ -700,7 +692,7 @@ LLVMToSPIRVDbgTran::transDbgTemplateParameter(const DITemplateParameter *TP) {
   using namespace SPIRVDebug::Operand::TemplateParameter;
   SPIRVWordVec Ops(OperandCount);
   Ops[NameIdx] = BM->getString(TP->getName())->getId();
-  Ops[TypeIdx] = transDbgEntryRef(TP->getType(), getVoidTy())->getId();
+  Ops[TypeIdx] = transDbgEntry(TP->getType())->getId();
   Ops[ValueIdx] = getDebugInfoNoneId();
   if (TP->getTag() == dwarf::DW_TAG_template_value_parameter) {
     const DITemplateValueParameter *TVP = cast<DITemplateValueParameter>(TP);
@@ -755,7 +747,7 @@ LLVMToSPIRVDbgTran::transDbgGlobalVariable(const DIGlobalVariable *GV) {
   using namespace SPIRVDebug::Operand::GlobalVariable;
   SPIRVWordVec Ops(MinOperandCount);
   Ops[NameIdx] = BM->getString(GV->getName())->getId();
-  Ops[TypeIdx] = transDbgEntryRef(GV->getType())->getId();
+  Ops[TypeIdx] = transDbgEntry(GV->getType())->getId();
   Ops[SourceIdx] = getSource(GV)->getId();
   Ops[LineIdx] = GV->getLine();
   Ops[ColumnIdx] = 0; // This version of DIGlobalVariable has no column number
@@ -925,7 +917,7 @@ LLVMToSPIRVDbgTran::transDbgLocalVariable(const DILocalVariable *Var) {
   SPIRVWordVec Ops(MinOperandCount);
 
   Ops[NameIdx] = BM->getString(Var->getName())->getId();
-  Ops[TypeIdx] = transDbgEntryRef(Var->getType())->getId();
+  Ops[TypeIdx] = transDbgEntry(Var->getType())->getId();
   Ops[SourceIdx] = getSource(Var->getFile())->getId();
   Ops[LineIdx] = Var->getLine();
   Ops[ColumnIdx] = 0; // This version of DILocalVariable has no column number
@@ -968,7 +960,7 @@ LLVMToSPIRVDbgTran::transDbgImportedEntry(const DIImportedEntity *IE) {
   Ops[NameIdx] = BM->getString(IE->getName())->getId();
   Ops[TagIdx] = SPIRV::DbgImportedEntityMap::map(Tag);
   Ops[SourceIdx] = getSource(IE->getFile())->getId();
-  Ops[EntityIdx] = transDbgEntryRef(IE->getEntity())->getId();
+  Ops[EntityIdx] = transDbgEntry(IE->getEntity())->getId();
   Ops[LineIdx] = IE->getLine();
   Ops[ColumnIdx] = 0; // This version of DIImportedEntity has no column number
   Ops[ParentIdx] = getScope(IE->getScope())->getId();
