@@ -51,7 +51,6 @@
 #include "llvm/Pass.h"
 #include "llvm/PassSupport.h"
 #include "llvm/Support/Debug.h"
-#include "llvm/Support/raw_ostream.h"
 
 #include <algorithm>
 #include <set>
@@ -833,9 +832,10 @@ void OCL20ToSPIRV::transAtomicBuiltin(CallInst *CI, OCLBuiltinTransInfo &Info) {
             return map<Scope>(static_cast<OCLScopeKind>(I));
           });
         } else {
-          // SPIR-V 1.3 r6 s2.16.2: All <id> used for Scope and Memory
-          // Semantics must be of an OpConstant.
-          Ctx->emitError(CI, "memory_scope argument needs to be constant");
+          Args[ScopeIdx] =
+              getOrCreateSwitchFunc(kSPIRVName::TranslateOCLMemScope,
+                                    Args[ScopeIdx], OCLMemScopeMap::getMap(),
+                                    false /*IsReverse*/, OCLMS_device, CI, M);
         }
         for (size_t I = 0; I < NumOrder; ++I) {
           if (auto OrderInt =
@@ -845,9 +845,10 @@ void OCL20ToSPIRV::transAtomicBuiltin(CallInst *CI, OCLBuiltinTransInfo &Info) {
                   0, static_cast<OCLMemOrderKind>(Ord));
             });
           } else {
-            // SPIR-V 1.3 r6 s2.16.2: All <id> used for Scope and Memory
-            // Semantics must be of an OpConstant.
-            Ctx->emitError(CI, "memory_order argument needs to be constant");
+            Args[OrderIdx + I] = getOrCreateSwitchFunc(
+                kSPIRVName::TranslateOCLMemOrder, Args[OrderIdx + I],
+                OCLMemOrderMap::getMap(), false /*IsReverse*/, OCLMO_seq_cst,
+                CI, M);
           }
         }
         // Order of args in SPIR-V:
@@ -1112,7 +1113,7 @@ void OCL20ToSPIRV::visitCallReadImageWithSampler(
           assert(0 && "read_image* with unhandled number of args!");
         }
 
-        // SPIR-V intruction always returns 4-element vector
+        // SPIR-V instruction always returns 4-element vector
         if (IsRetScalar)
           Ret = VectorType::get(Ret, 4);
         return getSPIRVFuncName(OpImageSampleExplicitLod,
@@ -1691,7 +1692,7 @@ void OCL20ToSPIRV::visitSubgroupImageMediaBlockINTEL(
   mutateCallInstSPIRV(
       M, CI,
       [=](CallInst *, std::vector<Value *> &Args) {
-        // Moving the last argument to the begining.
+        // Moving the last argument to the beginning.
         std::rotate(Args.begin(), Args.end() - 1, Args.end());
         return getSPIRVFuncName(OpCode, CI->getType());
       },
