@@ -51,7 +51,6 @@
 #include "llvm/PassSupport.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/Debug.h"
-#include "llvm/Support/raw_ostream.h"
 
 using namespace llvm;
 using namespace SPIRV;
@@ -115,6 +114,12 @@ size_t getAtomicBuiltinNumMemoryOrderArgs(StringRef Name) {
   return 1;
 }
 
+size_t getSPIRVAtomicBuiltinNumMemoryOrderArgs(Op OC) {
+  if (OC == OpAtomicCompareExchange || OC == OpAtomicCompareExchangeWeak)
+    return 2;
+  return 1;
+}
+
 bool isComputeAtomicOCLBuiltin(StringRef DemangledName) {
   if (!DemangledName.startswith(kOCLBuiltinName::AtomicPrefix) &&
       !DemangledName.startswith(kOCLBuiltinName::AtomPrefix))
@@ -193,18 +198,6 @@ unsigned getExtOp(StringRef OrigName, const std::string &GivenDemangledName) {
     return ~0U;
 }
 
-std::unique_ptr<SPIRVEntry> getSPIRVInst(const OCLBuiltinTransInfo &Info) {
-  Op OC = OpNop;
-  unsigned ExtOp = ~0U;
-  SPIRVEntry *Entry = nullptr;
-  if (OCLSPIRVBuiltinMap::find(Info.UniqName, &OC))
-    Entry = SPIRVEntry::create(OC);
-  else if ((ExtOp = getExtOp(Info.MangledName, Info.UniqName)) != ~0U)
-    Entry = static_cast<SPIRVEntry *>(
-        SPIRVEntry::createUnique(SPIRVEIS_OpenCL, ExtOp).get());
-  return std::unique_ptr<SPIRVEntry>(Entry);
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Functions for getting module info
@@ -430,6 +423,8 @@ static FunctionType *getBlockInvokeTy(Function *F, unsigned BlockIdx) {
 class OCLBuiltinFuncMangleInfo : public SPIRV::BuiltinFuncMangleInfo {
 public:
   OCLBuiltinFuncMangleInfo(Function *F) : F(F) {}
+  OCLBuiltinFuncMangleInfo(ArrayRef<Type *> ArgTypes)
+      : ArgTypes(ArgTypes.vec()) {}
   void init(const std::string &UniqName) override {
     UnmangledName = UniqName;
     size_t Pos = std::string::npos;
@@ -558,6 +553,15 @@ public:
       addVoidPtrArg(1);
       addUnsignedArg(2);
       addUnsignedArg(3);
+      // OpenCL-like representation of blocking pipes
+    } else if (UnmangledName == "read_pipe_2_bl" ||
+               UnmangledName == "write_pipe_2_bl") {
+      // with 2 arguments (plus two i32 literals):
+      // int read_pipe_bl (read_only pipe gentype p, gentype *ptr)
+      // int write_pipe_bl (write_only pipe gentype p, const gentype *ptr)
+      addVoidPtrArg(1);
+      addUnsignedArg(2);
+      addUnsignedArg(3);
     } else if (UnmangledName == "read_pipe_4" ||
                UnmangledName == "write_pipe_4") {
       // with 4 arguments (plus two i32 literals):
@@ -668,11 +672,40 @@ public:
           }
         }
       }
+    } else if (UnmangledName == "intel_sub_group_shuffle_down" ||
+               UnmangledName == "intel_sub_group_shuffle_up") {
+      addUnsignedArg(2);
+    } else if (UnmangledName == "intel_sub_group_shuffle" ||
+               UnmangledName == "intel_sub_group_shuffle_xor") {
+      addUnsignedArg(1);
+    } else if (UnmangledName.find("intel_sub_group_block_write") !=
+               std::string::npos) {
+      // distinguish write to image and other data types as position
+      // of uint argument is different though name is the same.
+      assert(ArgTypes.size() && "lack of necessary information");
+      if (ArgTypes[0]->isPointerTy() &&
+          ArgTypes[0]->getPointerElementType()->isIntegerTy()) {
+        addUnsignedArg(0);
+        addUnsignedArg(1);
+      } else {
+        addUnsignedArg(2);
+      }
+    } else if (UnmangledName.find("intel_sub_group_block_read") !=
+               std::string::npos) {
+      // distinguish read from image and other data types as position
+      // of uint argument is different though name is the same.
+      assert(ArgTypes.size() && "lack of necessary information");
+      if (ArgTypes[0]->isPointerTy() &&
+          ArgTypes[0]->getPointerElementType()->isIntegerTy()) {
+        setArgAttr(0, SPIR::ATTR_CONST);
+        addUnsignedArg(0);
+      }
     }
   }
-  // Auxiliarry information, it is expected what it is relevant at the moment
+  // Auxiliarry information, it is expected that it is relevant at the moment
   // the init method is called.
   Function *F; // SPIRV decorated function
+  std::vector<Type *> ArgTypes; // Arguments of OCL builtin
 };
 
 CallInst *mutateCallInstOCL(
@@ -692,14 +725,6 @@ Instruction *mutateCallInstOCL(
   return mutateCallInst(M, CI, ArgMutate, RetMutate, &BtnInfo, Attrs);
 }
 
-void mutateFunctionOCL(
-    Function *F,
-    std::function<std::string(CallInst *, std::vector<Value *> &)> ArgMutate,
-    AttributeList *Attrs) {
-  OCLBuiltinFuncMangleInfo BtnInfo(F);
-  return mutateFunction(F, ArgMutate, &BtnInfo, Attrs, false);
-}
-
 static std::pair<StringRef, StringRef>
 getSrcAndDstElememntTypeName(BitCastInst *BIC) {
   if (!BIC)
@@ -754,6 +779,7 @@ bool isSamplerTy(Type *Ty) {
 
 bool isPipeBI(const StringRef MangledName) {
   return MangledName == "write_pipe_2" || MangledName == "read_pipe_2" ||
+         MangledName == "write_pipe_2_bl" || MangledName == "read_pipe_2_bl" ||
          MangledName == "write_pipe_4" || MangledName == "read_pipe_4" ||
          MangledName == "reserve_write_pipe" ||
          MangledName == "reserve_read_pipe" ||
@@ -818,6 +844,6 @@ void checkFpContract(BinaryOperator *B, SPIRVBasicBlock *BB) {
 void llvm::mangleOpenClBuiltin(const std::string &UniqName,
                                ArrayRef<Type *> ArgTypes,
                                std::string &MangledName) {
-  OCLUtil::OCLBuiltinFuncMangleInfo BtnInfo(nullptr);
+  OCLUtil::OCLBuiltinFuncMangleInfo BtnInfo(ArgTypes);
   MangledName = SPIRV::mangleBuiltin(UniqName, ArgTypes, &BtnInfo);
 }
