@@ -41,10 +41,12 @@
 
 #include "SPIRVInternal.h"
 #include "llvm/IR/DebugInfoMetadata.h"
+#include "llvm/IR/IRBuilder.h"
 #include "llvm/Support/Path.h"
 
 #include <functional>
 #include <tuple>
+#include <type_traits>
 #include <utility>
 using namespace SPIRV;
 using namespace llvm;
@@ -64,6 +66,18 @@ enum OCLMemFenceKind {
   OCLMF_Image = 4,
 };
 
+// This enum declares extra constants for OpenCL mem_fence flag. It includes
+// combinations of local/global/image flags.
+enum OCLMemFenceExtendedKind {
+  OCLMFEx_Local = OCLMF_Local,
+  OCLMFEx_Global = OCLMF_Global,
+  OCLMFEx_Local_Global = OCLMF_Global | OCLMF_Local,
+  OCLMFEx_Image = OCLMF_Image,
+  OCLMFEx_Image_Local = OCLMF_Image | OCLMF_Local,
+  OCLMFEx_Image_Global = OCLMF_Image | OCLMF_Global,
+  OCLMFEx_Image_Local_Global = OCLMF_Image | OCLMF_Global | OCLMF_Local,
+};
+
 enum OCLScopeKind {
   OCLMS_work_item,
   OCLMS_work_group,
@@ -93,6 +107,9 @@ enum OCLMemOrderKind {
 
 typedef SPIRVMap<OCLMemFenceKind, MemorySemanticsMask> OCLMemFenceMap;
 
+typedef SPIRVMap<OCLMemFenceExtendedKind, MemorySemanticsMask>
+    OCLMemFenceExtendedMap;
+
 typedef SPIRVMap<OCLMemOrderKind, unsigned, MemorySemanticsMask> OCLMemOrderMap;
 
 typedef SPIRVMap<OCLScopeKind, Scope> OCLMemScopeMap;
@@ -105,6 +122,9 @@ typedef SPIRVMap<std::string, SPIRVFPRoundingModeKind>
 
 typedef SPIRVMap<std::string, Op, SPIRVInstruction> OCLSPIRVBuiltinMap;
 
+class OCL12Builtin;
+typedef SPIRVMap<std::string, Op, OCL12Builtin> OCL12SPIRVBuiltinMap;
+
 typedef SPIRVMap<std::string, SPIRVBuiltinVariableKind>
     SPIRSPIRVBuiltinVariableMap;
 
@@ -163,6 +183,7 @@ const static char Dot[] = "dot";
 const static char EnqueueKernel[] = "enqueue_kernel";
 const static char FMax[] = "fmax";
 const static char FMin[] = "fmin";
+const static char FPGARegIntel[] = "__builtin_intel_fpga_reg";
 const static char GetFence[] = "get_fence";
 const static char GetImageArraySize[] = "get_image_array_size";
 const static char GetImageChannelOrder[] = "get_image_channel_order";
@@ -183,6 +204,7 @@ const static char NDRangePrefix[] = "ndrange_";
 const static char Pipe[] = "pipe";
 const static char ReadImage[] = "read_image";
 const static char ReadPipe[] = "read_pipe";
+const static char ReadPipeBlockingINTEL[] = "read_pipe_bl";
 const static char RoundingPrefix[] = "_r";
 const static char Sampled[] = "sampled_";
 const static char SampledReadImage[] = "sampled_read_image";
@@ -204,6 +226,7 @@ const static char WaitGroupEvent[] = "wait_group_events";
 const static char WriteImage[] = "write_image";
 const static char WorkGroupBarrier[] = "work_group_barrier";
 const static char WritePipe[] = "write_pipe";
+const static char WritePipeBlockingINTEL[] = "write_pipe_bl";
 const static char WorkGroupPrefix[] = "work_group_";
 const static char WorkGroupAll[] = "work_group_all";
 const static char WorkGroupAny[] = "work_group_any";
@@ -291,9 +314,6 @@ const static char TypePrefix[] = "opencl.intel_sub_group_avc_";
 ///   function is translated to an extended instruction, otherwise ~0U.
 unsigned getExtOp(StringRef MangledName, const std::string &DemangledName = "");
 
-/// Get an empty SPIR-V instruction.
-std::unique_ptr<SPIRVEntry> getSPIRVInst(const OCLBuiltinTransInfo &Info);
-
 /// Get literal arguments of call of atomic_work_item_fence.
 AtomicWorkItemFenceLiterals getAtomicWorkItemFenceLiterals(CallInst *CI);
 
@@ -303,6 +323,9 @@ BarrierLiterals getBarrierLiterals(CallInst *CI);
 /// Get number of memory order arguments for atomic builtin function.
 size_t getAtomicBuiltinNumMemoryOrderArgs(StringRef Name);
 
+/// Get number of memory order arguments for spirv atomic builtin function.
+size_t getSPIRVAtomicBuiltinNumMemoryOrderArgs(Op OC);
+
 /// Return true for OpenCL builtins which do compute operations
 /// (like add, sub, min, max, inc, dec, ...) atomically
 bool isComputeAtomicOCLBuiltin(StringRef DemangledName);
@@ -383,12 +406,6 @@ Instruction *mutateCallInstOCL(
     std::function<Instruction *(CallInst *)> RetMutate,
     AttributeList *Attrs = nullptr);
 
-/// Mutate a function to OpenCL builtin function.
-void mutateFunctionOCL(
-    Function *F,
-    std::function<std::string(CallInst *, std::vector<Value *> &)> ArgMutate,
-    AttributeList *Attrs = nullptr);
-
 /// Check if instruction is bitcast from spirv.ConstantSampler to spirv.Sampler
 bool isSamplerInitializer(Instruction *Inst);
 
@@ -410,6 +427,13 @@ bool isSamplerTy(Type *Ty);
 // If so, it applies ContractionOff ExecutionMode to the kernel.
 void checkFpContract(BinaryOperator *B, SPIRVBasicBlock *BB);
 
+template <typename T> std::string toString(const T *Object) {
+  std::string S;
+  llvm::raw_string_ostream RSOS(S);
+  Object->print(RSOS);
+  RSOS.flush();
+  return S;
+}
 } // namespace OCLUtil
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -426,6 +450,22 @@ template <> inline void SPIRVMap<OCLMemFenceKind, MemorySemanticsMask>::init() {
   add(OCLMF_Image, MemorySemanticsImageMemoryMask);
 }
 
+template <>
+inline void SPIRVMap<OCLMemFenceExtendedKind, MemorySemanticsMask>::init() {
+  add(OCLMFEx_Local, MemorySemanticsWorkgroupMemoryMask);
+  add(OCLMFEx_Global, MemorySemanticsCrossWorkgroupMemoryMask);
+  add(OCLMFEx_Local_Global, MemorySemanticsWorkgroupMemoryMask |
+                                MemorySemanticsCrossWorkgroupMemoryMask);
+  add(OCLMFEx_Image, MemorySemanticsImageMemoryMask);
+  add(OCLMFEx_Image_Local,
+      MemorySemanticsWorkgroupMemoryMask | MemorySemanticsImageMemoryMask);
+  add(OCLMFEx_Image_Global,
+      MemorySemanticsCrossWorkgroupMemoryMask | MemorySemanticsImageMemoryMask);
+  add(OCLMFEx_Image_Local_Global, MemorySemanticsWorkgroupMemoryMask |
+                                      MemorySemanticsCrossWorkgroupMemoryMask |
+                                      MemorySemanticsImageMemoryMask);
+}
+
 template <>
 inline void SPIRVMap<OCLMemOrderKind, unsigned, MemorySemanticsMask>::init() {
   add(OCLMO_relaxed, MemorySemanticsMaskNone);
@@ -443,6 +483,58 @@ template <> inline void SPIRVMap<OCLScopeKind, Scope>::init() {
   add(OCLMS_sub_group, ScopeSubgroup);
 }
 
+template <class KeyTy, class ValTy, class Identifier = void>
+Instruction *
+getOrCreateSwitchFunc(StringRef MapName, Value *V,
+                      const SPIRVMap<KeyTy, ValTy, Identifier> &Map,
+                      bool IsReverse, Optional<int> DefaultCase,
+                      Instruction *InsertPoint, Module *M, int KeyMask = 0) {
+  static_assert(std::is_convertible<KeyTy, int>::value &&
+                    std::is_convertible<ValTy, int>::value,
+                "Can map only integer values");
+  Type *Ty = V->getType();
+  assert(Ty && Ty->isIntegerTy() && "Can't map non-integer types");
+  Function *F = getOrCreateFunction(M, Ty, Ty, MapName);
+  if (!F->empty()) // The switch function already exists. just call it.
+    return addCallInst(M, MapName, Ty, V, nullptr, InsertPoint);
+
+  F->setLinkage(GlobalValue::PrivateLinkage);
+
+  LLVMContext &Ctx = M->getContext();
+  BasicBlock *BB = BasicBlock::Create(Ctx, "entry", F);
+  IRBuilder<> IRB(BB);
+  SwitchInst *SI;
+  F->arg_begin()->setName("key");
+  if (KeyMask) {
+    Value *MaskV = ConstantInt::get(Type::getInt32Ty(Ctx), KeyMask);
+    Value *NewKey = IRB.CreateAnd(MaskV, F->arg_begin());
+    NewKey->setName("key.masked");
+    SI = IRB.CreateSwitch(NewKey, BB);
+  } else {
+    SI = IRB.CreateSwitch(F->arg_begin(), BB);
+  }
+
+  if (!DefaultCase) {
+    BasicBlock *DefaultBB = BasicBlock::Create(Ctx, "default", F);
+    IRBuilder<> DefaultIRB(DefaultBB);
+    DefaultIRB.CreateUnreachable();
+    SI->setDefaultDest(DefaultBB);
+  }
+
+  Map.foreach ([&](int Key, int Val) {
+    if (IsReverse)
+      std::swap(Key, Val);
+    BasicBlock *CaseBB = BasicBlock::Create(Ctx, "case." + Twine(Key), F);
+    IRBuilder<> CaseIRB(CaseBB);
+    CaseIRB.CreateRet(CaseIRB.getInt32(Val));
+    SI->addCase(IRB.getInt32(Key), CaseBB);
+    if (Key == DefaultCase)
+      SI->setDefaultDest(CaseBB);
+  });
+  assert(SI->getDefaultDest() != BB && "Invalid default destination in switch");
+  return addCallInst(M, MapName, Ty, V, nullptr, InsertPoint);
+}
+
 template <> inline void SPIRVMap<std::string, SPIRVGroupOperationKind>::init() {
   add("reduce", GroupOperationReduce);
   add("scan_inclusive", GroupOperationInclusiveScan);
@@ -583,6 +675,7 @@ template <> inline void SPIRVMap<std::string, Op, SPIRVInstruction>::init() {
   _SPIRV_OP(signbit, SignBitSet)
   _SPIRV_OP(any, Any)
   _SPIRV_OP(all, All)
+  _SPIRV_OP(popcount, BitCount)
   _SPIRV_OP(get_fence, GenericPtrMemSemantics)
   // CL 2.0 kernel enqueue builtins
   _SPIRV_OP(enqueue_marker, EnqueueMarker)
@@ -611,6 +704,8 @@ template <> inline void SPIRVMap<std::string, Op, SPIRVInstruction>::init() {
   // CL 2.0 pipe builtins
   _SPIRV_OP(read_pipe_2, ReadPipe)
   _SPIRV_OP(write_pipe_2, WritePipe)
+  _SPIRV_OP(read_pipe_2_bl, ReadPipeBlockingINTEL)
+  _SPIRV_OP(write_pipe_2_bl, WritePipeBlockingINTEL)
   _SPIRV_OP(read_pipe_4, ReservedReadPipe)
   _SPIRV_OP(write_pipe_4, ReservedWritePipe)
   _SPIRV_OP(reserve_read_pipe, ReserveReadPipePackets)
@@ -655,6 +750,22 @@ template <> inline void SPIRVMap<std::string, Op, SPIRVInstruction>::init() {
 #undef _SPIRV_OP
 }
 
+template <> inline void SPIRVMap<std::string, Op, OCL12Builtin>::init() {
+#define _SPIRV_OP(x, y) add(#x, Op##y);
+  _SPIRV_OP(add, AtomicIAdd)
+  _SPIRV_OP(sub, AtomicISub)
+  _SPIRV_OP(xchg, AtomicExchange)
+  _SPIRV_OP(cmpxchg, AtomicCompareExchange)
+  _SPIRV_OP(inc, AtomicIIncrement)
+  _SPIRV_OP(dec, AtomicIDecrement)
+  _SPIRV_OP(min, AtomicSMin)
+  _SPIRV_OP(max, AtomicSMax)
+  _SPIRV_OP(and, AtomicAnd)
+  _SPIRV_OP(or, AtomicOr)
+  _SPIRV_OP(xor, AtomicXor)
+#undef _SPIRV_OP
+}
+
 // SPV_INTEL_device_side_avc_motion_estimation extension builtins
 class SPIRVSubgroupsAVCIntelInst;
 template <>
