@@ -333,6 +333,11 @@ const static char ImageSampleExplicitLod[] = "ImageSampleExplicitLod";
 const static char ReservedPrefix[] = "reserved_";
 const static char SampledImage[] = "SampledImage";
 const static char TempSampledImage[] = "TempSampledImage";
+const static char TranslateOCLMemOrder[] = "__translate_ocl_memory_order";
+const static char TranslateOCLMemScope[] = "__translate_ocl_memory_scope";
+const static char TranslateSPIRVMemOrder[] = "__translate_spirv_memory_order";
+const static char TranslateSPIRVMemScope[] = "__translate_spirv_memory_scope";
+const static char TranslateSPIRVMemFence[] = "__translate_spirv_memory_fence";
 } // namespace kSPIRVName
 
 namespace kSPIRVPostfix {
@@ -414,7 +419,7 @@ public:
   void addUnsignedArg(int Ndx) { UnsignedArgs.insert(Ndx); }
   void addUnsignedArgs(int StartNdx, int StopNdx) {
     assert(StartNdx < StopNdx && "wrong parameters");
-    for (size_t I = StartNdx; I <= StopNdx; ++I)
+    for (int I = StartNdx; I <= StopNdx; ++I)
       addUnsignedArg(I);
   }
   void addVoidPtrArg(int Ndx) { VoidPtrArgs.insert(Ndx); }
@@ -524,9 +529,9 @@ inline size_t findFirstPtr(const std::vector<Value *> &Args) {
   return PtArg - Args.begin();
 }
 
-void removeFnAttr(LLVMContext *Context, CallInst *Call,
-                  Attribute::AttrKind Attr);
-void addFnAttr(LLVMContext *Context, CallInst *Call, Attribute::AttrKind Attr);
+bool isSupportedTriple(Triple T);
+void removeFnAttr(CallInst *Call, Attribute::AttrKind Attr);
+void addFnAttr(CallInst *Call, Attribute::AttrKind Attr);
 void saveLLVMModule(Module *M, const std::string &OutputFile);
 std::string mapSPIRVTypeToOCLType(SPIRVType *Ty, bool Signed);
 std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed);
