@@ -39,7 +39,6 @@
 // in this pass as a common functionality for both versions.
 //
 //===----------------------------------------------------------------------===//
-
 #define DEBUG_TYPE "spvtocl"
 
 #include "SPIRVToOCL.h"
@@ -55,7 +54,7 @@ static cl::opt<std::string>
 static cl::opt<std::string>
     OCLBuiltinsVersion("spirv-ocl-builtins-version",
                        cl::desc("Specify version of OCL builtins to translate "
-                                "to (CL1.0, CL1.1, CL1.2, CL2.0)"));
+                                "to (CL1.2, CL2.0, CL2.1)"));
 
 char SPIRVToOCL::ID = 0;
 
@@ -303,7 +302,8 @@ void SPIRVToOCL::visitCallSPIRVPipeBuiltin(CallInst *CI, Op OC) {
           Args.erase(Args.begin(), Args.begin() + 1);
 
         if (!(OC == OpReadPipe || OC == OpWritePipe ||
-              OC == OpReservedReadPipe || OC == OpReservedWritePipe))
+              OC == OpReservedReadPipe || OC == OpReservedWritePipe ||
+              OC == OpReadPipeBlockingINTEL || OC == OpWritePipeBlockingINTEL))
           return DemangledName;
 
         auto &P = Args[Args.size() - 3];
@@ -363,90 +363,14 @@ std::string SPIRVToOCL::getGroupBuiltinPrefix(CallInst *CI) {
   return Prefix;
 }
 
-Instruction *SPIRVToOCL::visitCallSPIRVAtomicBuiltin(CallInst *CI, Op OC) {
-  assert(CI->getCalledFunction() && "Unexpected indirect call");
-  AttributeList Attrs = CI->getCalledFunction()->getAttributes();
-  Instruction *PInsertBefore = CI;
-
-  return mutateCallInstOCL(
-      M, CI,
-      [=](CallInst *, std::vector<Value *> &Args, Type *&RetTy) {
-        auto Ptr = findFirstPtr(Args);
-        auto Name = OCLSPIRVBuiltinMap::rmap(OC);
-        auto NumOrder = getAtomicBuiltinNumMemoryOrderArgs(Name);
-        auto ScopeIdx = Ptr + 1;
-        auto OrderIdx = Ptr + 2;
-        if (OC == OpAtomicIIncrement || OC == OpAtomicIDecrement) {
-          // Since OpenCL 1.2 atomic_inc and atomic_dec builtins don't have,
-          // memory scope and memory order syntax, and OpenCL 2.0 doesn't have
-          // such builtins, therefore we translate these instructions to
-          // atomic_fetch_add_explicit and atomic_fetch_sub_explicit OpenCL 2.0
-          // builtins with "operand" argument = 1.
-          Name = OCLSPIRVBuiltinMap::rmap(
-              OC == OpAtomicIIncrement ? OpAtomicIAdd : OpAtomicISub);
-          Type *ValueTy =
-              cast<PointerType>(Args[Ptr]->getType())->getElementType();
-          assert(ValueTy->isIntegerTy());
-          Args.push_back(llvm::ConstantInt::get(ValueTy, 1));
-        }
-        Args[ScopeIdx] =
-            mapUInt(M, cast<ConstantInt>(Args[ScopeIdx]), [](unsigned I) {
-              return rmap<OCLScopeKind>(static_cast<Scope>(I));
-            });
-        for (size_t I = 0; I < NumOrder; ++I)
-          Args[OrderIdx + I] =
-              mapUInt(M, cast<ConstantInt>(Args[OrderIdx + I]),
-                      [](unsigned Ord) { return mapSPIRVMemOrderToOCL(Ord); });
-        std::swap(Args[ScopeIdx], Args.back());
-        if (OC == OpAtomicCompareExchange ||
-            OC == OpAtomicCompareExchangeWeak) {
-          // OpAtomicCompareExchange[Weak] semantics is different from
-          // atomic_compare_exchange_[strong|weak] semantics as well as
-          // arguments order.
-          // OCL built-ins returns boolean value and stores a new/original
-          // value by pointer passed as 2nd argument (aka expected) while SPIR-V
-          // instructions returns this new/original value as a resulting value.
-          AllocaInst *PExpected =
-              new AllocaInst(CI->getType(), 0, "expected",
-                             &(*PInsertBefore->getParent()
-                                    ->getParent()
-                                    ->getEntryBlock()
-                                    .getFirstInsertionPt()));
-          PExpected->setAlignment(CI->getType()->getScalarSizeInBits() / 8);
-          new StoreInst(Args[1], PExpected, PInsertBefore);
-          Args[1] = PExpected;
-          std::swap(Args[3], Args[4]);
-          std::swap(Args[2], Args[3]);
-          RetTy = Type::getInt1Ty(*Ctx);
-        }
-        return Name;
-      },
-      [=](CallInst *CI) -> Instruction * {
-        if (OC == OpAtomicCompareExchange ||
-            OC == OpAtomicCompareExchangeWeak) {
-          // OCL built-ins atomic_compare_exchange_[strong|weak] return boolean
-          // value. So, to obtain the same value as SPIR-V instruction is
-          // returning it has to be loaded from the memory where 'expected'
-          // value is stored. This memory must contain the needed value after a
-          // call to OCL built-in is completed.
-          LoadInst *POriginal =
-              new LoadInst(CI->getArgOperand(1), "original", PInsertBefore);
-          return POriginal;
-        }
-        // For other built-ins the return values match.
-        return CI;
-      },
-      &Attrs);
-}
-
 } // namespace SPIRV
 
 ModulePass *llvm::createSPIRVToOCL(Module &M) {
   if (OCLBuiltinsVersion.getNumOccurrences() > 0) {
-    std::string OCLVer = OCLBuiltinsVersion.getValue();
-    if (OCLVer == "CL1.0" || OCLVer == "CL1.1" || OCLVer == "CL1.2")
+    if (OCLBuiltinsVersion.getValue() == "CL1.2")
       return createSPIRVToOCL12();
-    else if (OCLVer == "CL2.0")
+    else if (OCLBuiltinsVersion.getValue() == "CL2.0" ||
+             OCLBuiltinsVersion.getValue() == "CL2.1")
       return createSPIRVToOCL20();
     else {
       assert(0 && "Invalid spirv-ocl-builtins-version value");
