@@ -56,7 +56,6 @@
 #include "llvm/Support/ErrorHandling.h"
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/ToolOutputFile.h"
-#include "llvm/Support/raw_ostream.h"
 
 #include <functional>
 #include <sstream>
@@ -78,12 +77,13 @@ cl::opt<bool, true> EnableDbgOutput("spirv-debug",
                                     cl::location(SPIRVDbgEnable));
 #endif
 
-void addFnAttr(LLVMContext *Context, CallInst *Call, Attribute::AttrKind Attr) {
+bool isSupportedTriple(Triple T) { return T.isSPIR(); }
+
+void addFnAttr(CallInst *Call, Attribute::AttrKind Attr) {
   Call->addAttribute(AttributeList::FunctionIndex, Attr);
 }
 
-void removeFnAttr(LLVMContext *Context, CallInst *Call,
-                  Attribute::AttrKind Attr) {
+void removeFnAttr(CallInst *Call, Attribute::AttrKind Attr) {
   Call->removeAttribute(AttributeList::FunctionIndex, Attr);
 }
 
@@ -656,7 +656,8 @@ Instruction *mutateCallInst(
   NewI->takeName(CI);
   NewI->setDebugLoc(CI->getDebugLoc());
   LLVM_DEBUG(dbgs() << " => " << *NewI << '\n');
-  CI->replaceAllUsesWith(NewI);
+  if (!CI->getType()->isVoidTy())
+    CI->replaceAllUsesWith(NewI);
   CI->eraseFromParent();
   return NewI;
 }
@@ -1237,7 +1238,7 @@ Type *getSPIRVTypeByChangeBaseTypeName(Module *M, Type *T, StringRef OldName,
   if (isSPIRVType(T, OldName, &Postfixes))
     return getOrCreateOpaquePtrType(M, getSPIRVTypeName(NewName, Postfixes));
   LLVM_DEBUG(dbgs() << " Invalid SPIR-V type " << *T << '\n');
-  llvm_unreachable("Invalid SPIRV-V type");
+  llvm_unreachable("Invalid SPIR-V type");
   return nullptr;
 }
 
@@ -1548,6 +1549,18 @@ spv::LoopControlMask getLoopControl(const BranchInst *Branch,
         Parameters.push_back(I);
         LoopControl |= spv::LoopControlPartialCountMask;
       }
+      if (S == "llvm.loop.ii.count") {
+        Parameters.push_back(InitiationIntervalINTEL);
+        size_t I = getMDOperandAsInt(Node, 1);
+        Parameters.push_back(I);
+        LoopControl |= spv::LoopControlExtendedControlsMask;
+      }
+      if (S == "llvm.loop.max_concurrency.count") {
+        Parameters.push_back(MaxConcurrencyINTEL);
+        size_t I = getMDOperandAsInt(Node, 1);
+        Parameters.push_back(I);
+        LoopControl |= spv::LoopControlExtendedControlsMask;
+      }
     }
   }
   return static_cast<spv::LoopControlMask>(LoopControl);
