@@ -40,6 +40,7 @@
 #ifndef SPIRV_LIBSPIRV_SPIRVENUM_H
 #define SPIRV_LIBSPIRV_SPIRVENUM_H
 
+#include "LLVMSPIRVOpts.h"
 #include "SPIRVOpCode.h"
 #include "spirv.hpp"
 #include <cstdint>
@@ -61,11 +62,6 @@ inline SPIRVWord mkWord(unsigned WordCount, Op OpCode) {
 
 const static unsigned KSpirvMemOrderSemanticMask = 0x1F;
 
-enum SPIRVVersion : SPIRVWord {
-  SPIRV_1_0 = 0x00010000,
-  SPIRV_1_1 = 0x00010100
-};
-
 enum SPIRVGeneratorKind {
   SPIRVGEN_KhronosLLVMSPIRVTranslator = 6,
   SPIRVGEN_KhronosSPIRVAssembler = 7,
@@ -113,18 +109,17 @@ typedef spv::GroupOperation SPIRVGroupOperationKind;
 typedef spv::Dim SPIRVImageDimKind;
 typedef std::vector<SPIRVCapabilityKind> SPIRVCapVec;
 
-enum SPIRVExtensionKind {
-  SPV_INTEL_device_side_avc_motion_estimation,
-  SPV_KHR_no_integer_wrap_decoration
-};
+typedef std::set<ExtensionID> SPIRVExtSet;
 
-typedef std::set<SPIRVExtensionKind> SPIRVExtSet;
-
-template <> inline void SPIRVMap<SPIRVExtensionKind, std::string>::init() {
-  add(SPV_INTEL_device_side_avc_motion_estimation,
-      "SPV_INTEL_device_side_avc_motion_estimation");
-  add(SPV_KHR_no_integer_wrap_decoration, "SPV_KHR_no_integer_wrap_decoration");
-};
+template <> inline void SPIRVMap<ExtensionID, std::string>::init() {
+#define _STRINGIFY(X) #X
+#define STRINGIFY(X) _STRINGIFY(X)
+#define EXT(X) add(ExtensionID::X, STRINGIFY(X));
+#include "LLVMSPIRVExtensions.inc"
+#undef EXT
+#undef STRINGIFY
+#undef _STRINGIFY
+}
 
 template <> inline void SPIRVMap<SPIRVExtInstSetKind, std::string>::init() {
   add(SPIRVEIS_OpenCL, "OpenCL.std");
@@ -156,6 +151,7 @@ template <> inline void SPIRVMap<SPIRVCapabilityKind, SPIRVCapVec>::init() {
   ADD_VEC_INIT(CapabilityImageReadWrite, {CapabilityImageBasic});
   ADD_VEC_INIT(CapabilityImageMipmap, {CapabilityImageBasic});
   ADD_VEC_INIT(CapabilityPipes, {CapabilityKernel});
+  ADD_VEC_INIT(CapabilityBlockingPipesINTEL, {CapabilityKernel});
   ADD_VEC_INIT(CapabilityDeviceEnqueue, {CapabilityKernel});
   ADD_VEC_INIT(CapabilityLiteralSampler, {CapabilityKernel});
   ADD_VEC_INIT(CapabilityAtomicStorage, {CapabilityShader});
@@ -351,6 +347,23 @@ template <> inline void SPIRVMap<Decoration, SPIRVCapVec>::init() {
   ADD_VEC_INIT(DecorationNoContraction, {CapabilityShader});
   ADD_VEC_INIT(DecorationInputAttachmentIndex, {CapabilityInputAttachment});
   ADD_VEC_INIT(DecorationAlignment, {CapabilityKernel});
+  ADD_VEC_INIT(DecorationRegisterINTEL, {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationMemoryINTEL, {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationNumbanksINTEL, {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationBankwidthINTEL, {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationMaxPrivateCopiesINTEL,
+               {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationSinglepumpINTEL,
+               {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationDoublepumpINTEL,
+               {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationMaxReplicatesINTEL,
+               {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationSimpleDualPortINTEL,
+               {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationMergeINTEL, {CapabilityFPGAMemoryAttributesINTEL});
+  ADD_VEC_INIT(DecorationReferencedIndirectlyINTEL,
+               {CapabilityIndirectReferencesINTEL});
 }
 
 template <> inline void SPIRVMap<BuiltIn, SPIRVCapVec>::init() {
