@@ -9,3 +9,5 @@ _SPIRV_OP(InvalidBuiltinSetName, "Expects OpenCL.std.")
 _SPIRV_OP(InvalidFunctionCall, "Unexpected llvm intrinsic:")
 _SPIRV_OP(InvalidArraySize, "Array size must be at least 1:")
 _SPIRV_OP(InvalidModule, "Invalid SPIR-V module:")
+_SPIRV_OP(UnimplementedOpCode, "Unimplemented opcode")
+_SPIRV_OP(FunctionPointers, "Can't translate function pointer:\n")
