@@ -124,7 +124,9 @@ public:
     SPIRVExtSet EV;
     if (!hasType())
       return EV;
-    return Type->getRequiredExtensions();
+    EV = Type->getRequiredExtensions();
+    assert(!Module || Module->isAllowedToUseExtensions(EV));
+    return EV;
   }
 
 protected:
