@@ -16,12 +16,12 @@ config.name = 'LLVM_SPIRV'
 config.test_format = lit.formats.ShTest(True)
 
 # suffixes: A list of file extensions to treat as test files.
-config.suffixes = ['.ll', '.spt']
+config.suffixes = ['.cl', '.ll', '.spt', '.spvasm']
 
 # excludes: A list of directories  and fles to exclude from the testsuite.
 config.excludes = ['CMakeLists.txt']
 
-if not config.skip_spirv_debug_info_tests:
+if not config.spirv_skip_debug_info_tests:
     # Direct object generation.
     config.available_features.add('object-emission')
     
@@ -41,13 +41,31 @@ config.test_exec_root = os.path.join(config.test_run_dir, 'test_output')
 
 llvm_config.use_default_substitutions()
 
-#llvm_config.use_clang()
+llvm_config.use_clang()
 
 config.substitutions.append(('%PATH%', config.environment['PATH']))
 
 tool_dirs = [config.llvm_tools_dir, config.llvm_spirv_dir]
 
-tools = ['llc', 'llvm-as', 'llvm-dis', 'llvm-dwarfdump', 'llvm-objdump', 'llvm-readelf', 'llvm-readobj',  'llvm-spirv']
+tools = ['llvm-as', 'llvm-dis', 'llvm-spirv', 'not']
+if not config.spirv_skip_debug_info_tests:
+    tools.extend(['llc', 'llvm-dwarfdump', 'llvm-objdump', 'llvm-readelf', 'llvm-readobj'])
 
 llvm_config.add_tool_substitutions(tools, tool_dirs)
 
+using_spirv_tools = False
+
+if config.spirv_tools_have_spirv_as:
+    llvm_config.add_tool_substitutions(['spirv-as'], [config.spirv_tools_bin_dir])
+    config.available_features.add('spirv-as')
+    using_spirv_tools = True
+
+if config.spirv_tools_have_spirv_val:
+    llvm_config.add_tool_substitutions(['spirv-val'], [config.spirv_tools_bin_dir])
+    using_spirv_tools = True
+else:
+    config.substitutions.append(('spirv-val', ':'))
+
+if using_spirv_tools:
+    new_ld_library_path = os.path.pathsep.join((config.spirv_tools_lib_dir, config.environment['LD_LIBRARY_PATH']))
+    config.environment['LD_LIBRARY_PATH'] = new_ld_library_path
